// Copyright (c) 2024 Infineon Technologies AG. All rights reserved
// Licensed under the Apache License, Version 2.0, see APACHE_LICENSE for details.
// SPDX-License-Identifier: Apache-2.0

#include "cy_pdl.h"
#include "cybsp.h"

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

/*******************************************************************************
* Macros
********************************************************************************/
#define BLINKY_TASK_NAME            ("Blinky")
#define BLINKY_TASK_STACK_SIZE      (configMINIMAL_STACK_SIZE)
#define BLINKY_TASK_PRIORITY        (tskIDLE_PRIORITY + 1)
#define MAIN_TASK_NAME              ("Main")
#define MAIN_TASK_STACK_SIZE        (configMINIMAL_STACK_SIZE)
#define MAIN_TASK_PRIORITY          (tskIDLE_PRIORITY + 1)

/* USER LED toggle period in milliseconds */
#define USER_LED_TOGGLE_PERIOD_MS   1000u

/*******************************************************************************
* Function Prototypes
********************************************************************************/
static void blinky_task(void *pvParameters);
static void main_task(void *pvParameters);


/*******************************************************************************
* Global Variables
********************************************************************************/
static SemaphoreHandle_t xSemaphore;

/*******************************************************************************
* Function Name: main
********************************************************************************
* Summary:
*  This is the main function. It creates two tasks, initializes the semaphore
*  for synchronization between tasks, and starts the FreeRTOS scheduler.
*
* Return: int
*
*******************************************************************************/
int main(void)
{
    /* Initialize the device and board peripherals */
    cy_rslt_t result = cybsp_init() ;
    if (result != CY_RSLT_SUCCESS)
    {
        CY_ASSERT(0);
    }
    
    // FreeRTOS assumes the clock source of the SysTick timer is the same as for the CPU
    // Set configSYSTICK_CLOCK_HZ value in FreeRTOSConfig.h depending on selected clock source 
    // if the selected clock source is not the CPU clock source.
    Cy_SysTick_SetClockSource(CY_SYSTICK_CLOCK_SOURCE_CLK_CPU);
        
    /* Create the Semaphore for synchronization between Blinky and Main task */
    xSemaphore = xSemaphoreCreateBinary();
    if( xSemaphore == NULL )
    {
        CY_ASSERT(0);
    }

    /* Create the RTOS tasks */
    BaseType_t retval = xTaskCreate(blinky_task, BLINKY_TASK_NAME, BLINKY_TASK_STACK_SIZE, NULL, BLINKY_TASK_PRIORITY, NULL);
    if (retval != pdPASS)
    {
        CY_ASSERT(0);
    }

    retval = xTaskCreate(main_task, MAIN_TASK_NAME, MAIN_TASK_STACK_SIZE, NULL, MAIN_TASK_PRIORITY, NULL);
    if (retval != pdPASS)
    {
        CY_ASSERT(0);
    }

    /* Start the scheduler */
    vTaskStartScheduler();

    for(;;)
    {
        /* vTaskStartScheduler never returns */
    }		
}

/*******************************************************************************
* Function Name: blinky_task
********************************************************************************
* Summary:
*  This RTOS task toggles the User LED each time the semaphore is obtained.
*
* Parameters:
*  void *pvParameters : Task parameter defined during task creation (unused)
*
* Return:
*  The RTOS task never returns.
*
*******************************************************************************/
static void blinky_task(void *pvParameters)
{
    (void) pvParameters;

    for(;;)
    {
        /* Block until the semaphore is given */
        xSemaphoreTake(xSemaphore, portMAX_DELAY);

        /* Toggle the USER LED state */
        Cy_GPIO_Inv(LED1_PORT, LED1_PIN);
    }
}

/*******************************************************************************
* Function Name: main_task
********************************************************************************
* Summary:
*  This RTOS task releases the semaphore every USER_LED_TOGGLE_PERIOD_MS.
*
* Parameters:
*  void *pvParameters : Task parameter defined during task creation (unused)
*
* Return:
*  The RTOS task never returns.
*
*******************************************************************************/
static void main_task(void *pvParameters)
{
    (void) pvParameters;
    const TickType_t xDelay = USER_LED_TOGGLE_PERIOD_MS / portTICK_PERIOD_MS;
    
    for(;;)
    {
        /* Block task for USER_LED_TOGGLE_PERIOD_MS. */
        vTaskDelay(xDelay);

        /* Release semaphore */
        xSemaphoreGive(xSemaphore);
    }
}

#if ( configCHECK_FOR_STACK_OVERFLOW > 0 )

void vApplicationStackOverflowHook(TaskHandle_t xTask,
                                   char * pcTaskName)
{
    /* Check pcTaskName for the name of the offending task,
     * or pxCurrentTCB if pcTaskName has itself been corrupted. */
    (void)xTask;
    (void)pcTaskName;
}

#endif /* #if ( configCHECK_FOR_STACK_OVERFLOW > 0 ) */

/* [] END OF FILE */
