// Copyright (c) 2024 Infineon Technologies AG. All rights reserved
// Licensed under the Apache License, Version 2.0, see APACHE_LICENSE for details.
// SPDX-License-Identifier: Apache-2.0

#include "cy_pdl.h"
#include "cybsp.h"

/*******************************************************************************
* Macros
********************************************************************************/
#define GPIO_INTERRUPT_PRIORITY (3u)

/*******************************************************************************
* Function Prototypes
********************************************************************************/
static void button_interrupt_handler(void);

/*******************************************************************************
* Global Variables
********************************************************************************/
static const cy_stc_sysint_t button1_intrCfg =
{
    .intrSrc = ((NvicMux3_IRQn << 16) | CYBSP_USER_BTN1_IRQ),
    .intrPriority = GPIO_INTERRUPT_PRIORITY
};

static const cy_stc_sysint_t button2_intrCfg =
{
    .intrSrc = ((NvicMux3_IRQn << 16) | CYBSP_USER_BTN2_IRQ),
    .intrPriority = GPIO_INTERRUPT_PRIORITY
};

static volatile bool button1_intr_flag = false;
static volatile bool button2_intr_flag = false;

/*******************************************************************************
* Function Name: main
********************************************************************************
* Summary:
* System entrance point. This function configures and initializes the GPIO
* interrupt, blink LED1 and LED2 on every GPIO interrupt generated by BUT1 and 
* BUT2 respectively.
*
* Return: int
*
*******************************************************************************/
int main(void)
{
    /* Initialize the device and board peripherals */
    cy_rslt_t result = cybsp_init();
    if (result != CY_RSLT_SUCCESS)
    {
        CY_ASSERT(0);
    }
    
    /* Enable global interrupts */
    __enable_irq();

    /* Configure GPIO interrupt vector */
    Cy_SysInt_Init(&button1_intrCfg, button_interrupt_handler);
    NVIC_ClearPendingIRQ((IRQn_Type)button1_intrCfg.intrSrc);
    
    Cy_SysInt_Init(&button2_intrCfg, button_interrupt_handler);
    NVIC_ClearPendingIRQ((IRQn_Type)button2_intrCfg.intrSrc);
    
    NVIC_EnableIRQ((IRQn_Type) NvicMux3_IRQn);
        
    while (1)
    {
        /* Check the interrupt status of BUT1*/
        if (button1_intr_flag)
        {
            button1_intr_flag = false;
            Cy_GPIO_Inv(CYBSP_USER_LED1_PORT, CYBSP_USER_LED1_PIN);
        }
        
        /* Check the interrupt status of BUT2*/
        if (button2_intr_flag)
        {
            button2_intr_flag = false;
            Cy_GPIO_Inv(CYBSP_USER_LED2_PORT, CYBSP_USER_LED2_PIN);
        }
    }
        
}

/*******************************************************************************
* Function Name: button_interrupt_handler
********************************************************************************
* Summary:
* GPIO interrupt handler.
*
* Return: int
*
*******************************************************************************/
static void button_interrupt_handler(void)
{
    if (1UL == Cy_GPIO_GetInterruptStatus(CYBSP_USER_BTN1_PORT, CYBSP_USER_BTN1_PIN))
    {
        Cy_GPIO_ClearInterrupt(CYBSP_USER_BTN1_PORT, CYBSP_USER_BTN1_PIN);
        button1_intr_flag = true;
    }
        
    if (1UL == Cy_GPIO_GetInterruptStatus(CYBSP_USER_BTN2_PORT, CYBSP_USER_BTN2_PIN))
    {
        Cy_GPIO_ClearInterrupt(CYBSP_USER_BTN2_PORT, CYBSP_USER_BTN2_PIN);
        button2_intr_flag = true;
    }		  
}